From 81e181ae44c9343b0772a8f064d007df23a8e35a Mon Sep 17 00:00:00 2001
From: mkirsz <s1351949@sms.ed.ac.uk>
Date: Tue, 26 Nov 2024 12:26:35 +0000
Subject: [PATCH 1/3] new add method

---
 include/tadah/mlip/st_descriptors_db.h | 19 +++++++++++--------
 src/st_descriptors_db.cpp              |  3 +++
 2 files changed, 14 insertions(+), 8 deletions(-)

diff --git a/src/st_descriptors_db.cpp b/src/st_descriptors_db.cpp
index 9f80ce0..21960ac 100644
--- a/src/st_descriptors_db.cpp
+++ b/src/st_descriptors_db.cpp
@@ -15,3 +15,6 @@ StDescriptors &StDescriptorsDB::operator()(const size_t s) {
 size_t StDescriptorsDB::size() const {
     return st_descs.size();
 }
+void StDescriptorsDB::add(const StDescriptors &st_d) {
+  st_descs.push_back(st_d);
+}
-- 
GitLab


From 9c11cfeea09d9130c79162428e20e35ab72d1e3b Mon Sep 17 00:00:00 2001
From: mkirsz <s1351949@sms.ed.ac.uk>
Date: Tue, 26 Nov 2024 13:10:09 +0000
Subject: [PATCH 2/3] replaced std::tmpnam

---
 tests/test_structure.cpp | 115 +++++++++++++++++++++++----------------
 1 file changed, 68 insertions(+), 47 deletions(-)

diff --git a/tests/test_structure.cpp b/tests/test_structure.cpp
index 055f39a..0b890d5 100644
--- a/tests/test_structure.cpp
+++ b/tests/test_structure.cpp
@@ -56,63 +56,84 @@ TEST_CASE( "Testing Structure virial pressure calculations", "[structure_virial_
   REQUIRE(fac*st.get_virial_pressure() == Approx(499.93));
 
 }
-TEST_CASE( "Testing Structure read and write", "[structure_read_write]" ) {
-  Structure st;
-  st.read("tests_data/structure_1.dat");
+TEST_CASE("Testing Structure read and write", "[structure_read_write]") {
+    Structure st;
+    st.read("tests_data/structure_1.dat");
+
+    REQUIRE(st.get_volume() == Approx(989.521812));
 
-  REQUIRE(st.get_volume() == Approx(989.521812));
+    REQUIRE(fac * st.get_virial_pressure() == Approx(26.705578));
+    REQUIRE(fac * st.get_pressure(300) == Approx(28.965914));
+    REQUIRE(fac * st.get_pressure(0) == Approx(26.705578));
 
-  REQUIRE(fac*st.get_virial_pressure() == Approx(26.705578));
-  REQUIRE(fac*st.get_pressure(300) == Approx(28.965914));
-  REQUIRE(fac*st.get_pressure(0) == Approx(26.705578));
+    // Create a temporary file
+    std::FILE* tmpFile = std::tmpfile();
+    if (!tmpFile) {
+        FAIL("Unable to create temporary file");
+    }
 
-  std::string tempfile = std::tmpnam(nullptr);
-  st.save(tempfile);
+    const int fd = fileno(tmpFile);
+    REQUIRE(fd != -1);
 
-  Structure st_temp;
-  st_temp.read(tempfile);
+    char tempfile[256];
+    snprintf(tempfile, sizeof(tempfile), "/proc/self/fd/%d", fd);
 
-  REQUIRE(st_temp.get_volume() == Approx(989.521812));
+    st.save(tempfile);
 
-  REQUIRE(fac*st_temp.get_virial_pressure() == Approx(26.705578));
-  REQUIRE(fac*st_temp.get_pressure(300) == Approx(28.965914));
-  REQUIRE(fac*st_temp.get_pressure(0) == Approx(26.705578));
+    Structure st_temp;
+    st_temp.read(tempfile);
 
-  REQUIRE(st==st_temp);
+    REQUIRE(st_temp.get_volume() == Approx(989.521812));
+    REQUIRE(fac * st_temp.get_virial_pressure() == Approx(26.705578));
+    REQUIRE(fac * st_temp.get_pressure(300) == Approx(28.965914));
+    REQUIRE(fac * st_temp.get_pressure(0) == Approx(26.705578));
 
-  std::remove(tempfile.c_str());
+    REQUIRE(st == st_temp);
 
+    // Temporary file will be closed automatically
 }
-TEST_CASE( "Testing Structure compare", "[structure_compare]" ) {
-  Structure st;
-  st.read("tests_data/structure_1.dat");
-  std::string tempfile = std::tmpnam(nullptr);
-  st.save(tempfile);
-
-  Structure st_temp;
-  st_temp.read(tempfile);
-
-  SECTION("Compare unchanged") {
-    REQUIRE(st==st_temp);
-  }
-  SECTION("Compare symbols") {
-    REQUIRE(st==st_temp);
-    st_temp.atoms[0].symbol[0]='X';
-    st_temp.atoms[0].symbol[1]='X';
-    REQUIRE(!(st==st_temp));
-  }
-  SECTION("Compare position") {
-    REQUIRE(st==st_temp);
-    st_temp.atoms[0].position(0.12,0.13,10.14);
-    REQUIRE(!(st==st_temp));
-  }
-  SECTION("Compare force") {
-    REQUIRE(st==st_temp);
-    st_temp.atoms[0].force(1.12, 0.13, 0.134);
-    REQUIRE(!(st==st_temp));
-  }
-
-  std::remove(tempfile.c_str());
+TEST_CASE("Testing Structure compare", "[structure_compare]") {
+    Structure st;
+    st.read("tests_data/structure_1.dat");
+
+    // Create a temporary file
+    std::FILE* tmpFile = std::tmpfile();
+    if (!tmpFile) {
+        FAIL("Unable to create temporary file");
+    }
+
+    const int fd = fileno(tmpFile);
+    REQUIRE(fd != -1);
+
+    char tempfile[256];
+    snprintf(tempfile, sizeof(tempfile), "/proc/self/fd/%d", fd);
+
+    st.save(tempfile);
+
+    Structure st_temp;
+    st_temp.read(tempfile);
+
+    SECTION("Compare unchanged") {
+        REQUIRE(st == st_temp);
+    }
+    SECTION("Compare symbols") {
+        REQUIRE(st == st_temp);
+        st_temp.atoms[0].symbol[0] = 'X';
+        st_temp.atoms[0].symbol[1] = 'X';
+        REQUIRE(!(st == st_temp));
+    }
+    SECTION("Compare position") {
+        REQUIRE(st == st_temp);
+        st_temp.atoms[0].position(0.12, 0.13, 10.14);
+        REQUIRE(!(st == st_temp));
+    }
+    SECTION("Compare force") {
+        REQUIRE(st == st_temp);
+        st_temp.atoms[0].force(1.12, 0.13, 0.134);
+        REQUIRE(!(st == st_temp));
+    }
+
+    // Temporary file will be closed automatically
 }
 TEST_CASE( "Testing Structure copy", "[structure_copy]" ) {
   Structure st;
-- 
GitLab


From 81a3bb2bce791926165bbc7e5f9010866deabedc Mon Sep 17 00:00:00 2001
From: mkirsz <s1351949@sms.ed.ac.uk>
Date: Tue, 26 Nov 2024 13:10:39 +0000
Subject: [PATCH 3/3] Added range based loops

---
 include/tadah/mlip/st_descriptors_db.h |  6 ++++++
 src/st_descriptors_db.cpp              | 15 +++++++++++++++
 2 files changed, 21 insertions(+)

diff --git a/include/tadah/mlip/st_descriptors_db.h b/include/tadah/mlip/st_descriptors_db.h
index 7f45999..6272db8 100644
--- a/include/tadah/mlip/st_descriptors_db.h
+++ b/include/tadah/mlip/st_descriptors_db.h
@@ -38,5 +38,11 @@ struct StDescriptorsDB {
   /** Add StDescriptors to DB */
   void add(const StDescriptors &st_d);
 
+  // Methods to enable range-based for loop
+  std::vector<StDescriptors>::iterator begin();
+  std::vector<StDescriptors>::iterator end();
+  std::vector<StDescriptors>::const_iterator begin() const;
+  std::vector<StDescriptors>::const_iterator end() const;
+
 };
 #endif
diff --git a/src/st_descriptors_db.cpp b/src/st_descriptors_db.cpp
index 21960ac..3dce74f 100644
--- a/src/st_descriptors_db.cpp
+++ b/src/st_descriptors_db.cpp
@@ -18,3 +18,18 @@ size_t StDescriptorsDB::size() const {
 void StDescriptorsDB::add(const StDescriptors &st_d) {
   st_descs.push_back(st_d);
 }
+std::vector<StDescriptors>::iterator StDescriptorsDB::begin() { 
+    return st_descs.begin(); 
+}
+
+std::vector<StDescriptors>::iterator StDescriptorsDB::end() { 
+    return st_descs.end(); 
+}
+
+std::vector<StDescriptors>::const_iterator StDescriptorsDB::begin() const { 
+    return st_descs.cbegin(); 
+}
+
+std::vector<StDescriptors>::const_iterator StDescriptorsDB::end() const { 
+    return st_descs.cend(); 
+}
diff --git a/include/tadah/mlip/st_descriptors_db.h b/include/tadah/mlip/st_descriptors_db.h
index db8843c..7f45999 100644
--- a/include/tadah/mlip/st_descriptors_db.h
+++ b/include/tadah/mlip/st_descriptors_db.h
@@ -15,25 +15,28 @@
  *
  */
 struct StDescriptorsDB {
-    std::vector<StDescriptors> st_descs;
-    StDescriptorsDB() {};
+  std::vector<StDescriptors> st_descs;
+  StDescriptorsDB() {};
 
-    /** This constructor fully initialise this object
+  /** This constructor fully initialise this object
      *
      * Requires:
      * - StructureDB st to have all nearest neighbours calculated
      * - Config to contain keys: \ref FORCE, \ref STRESS
      *   and \ref INTERNAL_KEY \ref DSIZE
      */
-    StDescriptorsDB(const StructureDB &stdb, Config &config);
+  StDescriptorsDB(const StructureDB &stdb, Config &config);
 
-    /** Return reference to the n-th StDescriptors
+  /** Return reference to the n-th StDescriptors
      * stored by this object
      */
-    StDescriptors& operator()(const size_t n);
+  StDescriptors& operator()(const size_t n);
 
-    /** Return number of StDescriptors stored */
-    size_t size() const;
+  /** Return number of StDescriptors stored */
+  size_t size() const;
+
+  /** Add StDescriptors to DB */
+  void add(const StDescriptors &st_d);
 
 };
 #endif
-- 
GitLab